#include "libraries.h"
#include "functions.h"

#include "StickDude.h"

#include "SSD1306.h"

////////////////////////////////////////////////////////////////////////////////
// This code is based on using an
// IZOKEE 0.96" SSD1306 based 128x64 OLED (Yellow/Blue) from Amazon
// { https://www.amazon.ca/gp/product/B076PNP2VD/ }
////////////////////////////////////////////////////////////////////////////////

// Initialize the display
void SSD1306_init()
{
  Serial.printf("+---------------------------------------+\n");
  Wire.begin(SSD1306_SDA, SSD1306_SCL); // Select SDA & SCL pins

  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C))
  {
    Serial.printf("| SSD1306 allocation failed             |\n");
    for (;;)
      ;   // Don't proceed, loop forever
          //  Does this ever actually happen?
  }

  display.clearDisplay();

  display.dim(true); // These displays are excessively bright

  // Set up the active (changing) parts of the display
  Dcanvas1.setFont(&FreeMonoBold9pt7b);
  Dcanvas2.setFont(&FreeMonoBold9pt7b);
  // Dcanvas3.setFont(&FreeMonoBold9pt7b);

  // Set up the static (non-changing) parts of the display
  // Scanvas1.setFont(&FreeMonoBold9pt7b);
  // Scanvas2.setFont(&FreeMonoBold9pt7b);
  Scanvas3.setFont(&FreeMonoBold9pt7b);

  display.setTextColor(WHITE, BLACK);
  display.setTextSize(1);

  Serial.printf("| SSD1306 initialized                   |\n");
}

// Place static text on the yellow part of the OLED display
// Accepts the string (max 9 characters)
void SSD1306_title(const char *Title)
{ // Max 9 characters...
  display.setFont(&FreeMonoBold12pt7b);
  display.setCursor(0, 14);
  display.print(Title);
  display.display();
}

// Place static text on the blue part of the OLED display
// Accepts the string (max 11 characters) and the row number (1-3)
void SSD1306_Static(const char *Line, int Row)
{ // Max 11 characters
  Scanvas3.fillScreen(BLACK);
  Scanvas3.setCursor(0, 11);
  Scanvas3.print(Line);
  display.drawBitmap(4, row_Y[Row - 1] - 9, Scanvas3.getBuffer(), 120, 15, WHITE, BLACK);

  display.display();
}

// Create an activity indicator
// Just a little dot that crawls across the bottom edge of the display
void SSD1306_Dotty()
{
  DottyX++;
  if (DottyX > SCREEN_WIDTH)
  {
    DottyX = 0;
  }
  // display.drawPixel(DottyX, SCREEN_HEIGHT - 1, WHITE); // Bottom line
  // display.drawPixel(DottyX - 1, SCREEN_HEIGHT - 1, BLACK);
  // display.drawBitmap(DottyX, 64 - 11, StickDude.getBuffer(), 8, 11, WHITE, BLACK);
  testStickDude(0, 0, 0);
  display.drawBitmap(DottyX, 64 - 12, StickDudePIC.getBuffer(), 8, 12, WHITE, BLACK);
  display.display();
}

// Display the temperatures on the pre-formatted OLED display
void SSD1306_DisplayTemps()
{
  float Beer = DS18B20_getReading(0); // * .3;
  float Food = DS18B20_getReading(1); // * -1;

  char BeerTemp[6];
  char FoodTemp[6];

  Dcanvas1.fillScreen(BLACK);
  if (Beer == -127)
  {
    sprintf(BeerTemp, "-----");
    Beer = 0;
  }
  else
    sprintf(BeerTemp, "%5.1f", Beer);

  Dcanvas2.fillScreen(BLACK);
  if (Food == -127)
  {
    sprintf(FoodTemp, "-----");
    Food = 0;
  }
  else
    sprintf(FoodTemp, "%5.1f", Food);

  Dcanvas1.setCursor(0, 11);
  Dcanvas1.print(BeerTemp);
  display.drawBitmap(64, row_Y[0] - 9, Dcanvas1.getBuffer(), 55, 12, WHITE, BLACK);

  Dcanvas2.setCursor(0, 11);
  Dcanvas2.print(FoodTemp);
  display.drawBitmap(64, row_Y[1] - 9, Dcanvas2.getBuffer(), 55, 12, WHITE, BLACK);

  display.display();

  MQTT_SendData(Beer, Food);

  waveStickDude(0, 0);

#ifdef DEBUG
  Serial.printf("| Sensor 0: %5s - Sensor 1: %5s     |\n", BeerTemp, FoodTemp);
#endif
}

void testStickDude(int DudeNum, int TOP, int LEFT)
{
  // display.clearDisplay();
  StickDudePIC.fillScreen(BLACK);

  StickDudePIC.drawBitmap(
      (0),
      (0),
      StickDude[DudeNum], DUDE_WIDTH, DUDE_HEIGHT, 1);

  // display.display();
}

void walkStickDude(int DudeNum, int TOP, int LEFT)
{
  display.clearDisplay();

  display.drawBitmap(
      (LEFT),
      (TOP),
      StickDudeWalk[DudeNum], DUDE_WIDTH, DUDE_HEIGHT, 1);

  display.display();
}

void waveStickDude(int TOP, int LEFT)
{
  Serial.printf("| Loop through StickDudes Poses...      |\n");
  int FOO;

  for (FOO = 0; FOO < DUDE_COUNT; FOO++)
  {
    {
      testStickDude(FOO, 0, 0);
  display.drawBitmap(DottyX, 64 - 12, StickDudePIC.getBuffer(), 8, 12, WHITE, BLACK);
  display.display();
      delay(20);
    }
  }
  FOO--;
  for (; FOO >= 0; FOO--)
  {
    {
      testStickDude(FOO, 0, 0);
  display.drawBitmap(DottyX, 64 - 12, StickDudePIC.getBuffer(), 8, 12, WHITE, BLACK);
  display.display();
      delay(20);
    }
  }
}
